//-------------------------------------------------------------------------------------------
// Project            ForeignControls.palette
// Class              ForeignCodeAssociation
// Creator            Christian Rapp
// Maintainer         Christian Rapp
// Creation Date      Tue Oct 10 15:45:29 MET 1995
//
// Copyright (C) Uptime Object Factory Inc - All rights reserved.
//
// $Id: ForeignCodeAssociation.h,v 1.1.1.1 1996/03/18 12:53:54 cr Exp $
//-------------------------------------------------------------------------------------------
/* Description:
A ForeignCodeAssociation uses a ForeignControl subclass (ForeignTextField or ForeignPopUp) to display the value of an enterprise object for its key, and notifies its controller of any user changes. 
The ForeignCodeAssociation is especially useful to form a join to a small enterprise object. To set up a ForeignCodeAssociation, you have to create a ForeignTextField or ForeignPopUp instance in Interface Builder. You then form the association from the controller to the ForeignControl subclass. In the connection inspector select an attribute from the entity, you want to connect to.
When the association is notified of a change by its controller, it gets the value of the selected enterprise object and sets it as codeValue in the ForeignControl subclass. This forces a fetch of the corresponding enterprise object. If the controller has an empty or a multiple selection, the ForeignCodeAssociation instance set objectValue to @"".
When the user changes something in the ForeignControl subclass, the association simply sends an associationDidEdit: message to its controller. An ForeignCodeAssociation value is a NSString object.
*/
//-------------------------------------------------------------------------------------------

#import "ForeignControlsGlobal.h"
#import "ForeignControlDelegate.h"
#import <eointerface/eointerface.h>


@interface ForeignCodeAssociation : EOAssociation <ForeignControlDelegate>
{
}

//===========================================================================================
//==== ForeignControlDelegate protocol
//===========================================================================================

- (BOOL)controlWillChange:sender;
	// This delegate method is invoked from the foreign control if the control will change its object value

- (void)controlDidChange:sender;
	// This delegate method is invoked from the foreign control if the control did change its object value

//===========================================================================================
//==== Initializing instances
//===========================================================================================

- initWithController:controller key:key destination:destination;
	// Initializes the association, sets itself as the delegate of the ForeignControl subclass to make sure that the association notifies the changes.
	//
	// See also:  - controlWillChange:, - controlDidChange:
	
//===========================================================================================
//==== Methods invoked by the EOController
//===========================================================================================

- (void)contentsDidChange;
	// Is invoked by the controller and invokes _update.
	
- (void)selectionDidChange;
	// Is invoked by the controller and invokes _update.
	
- value;
	// Returns the codeValue (NSString *) of the ForeignControl subclass. If the codeValue is @"", then an EONull instance is returned.

//===========================================================================================
//==== Internal methods
//===========================================================================================

- (void)_update;
	// Gets the value of the selected enterprise object and sets it as codeValue in the ForeignControl subclass. If the controller has an empty or a multiple selection, the ForeignObjectAssociation instance set codeValue to @"".

- (BOOL)startEditing;
	// Not yet documented.
		
@end
