//-------------------------------------------------------------------------------------------
// Project            ForeignControls.palette
// Class              ForeignObjectAssociation
// Creator            Christian Rapp
// Maintainer         Christian Rapp
// Creation Date      Tue Oct 10 15:45:29 MET 1995
//
// Copyright (C) Uptime Object Factory Inc - All rights reserved.
//
// $Id: ForeignObjectAssociation.h,v 1.1.1.1 1996/03/18 12:53:55 cr Exp $
//-------------------------------------------------------------------------------------------
/* Description:
An ForeignObjectAssociation uses a ForeignControl subclass (ForeignTextField or ForeignPopUp) to display two values (a main value and a code value) of an enterprise object and it notifies its controller when the user made a change in the ForeignControl subclass. 
The ForeignObjectAssociation was developed to control joins between big enterprise objects which are connected with an "to one". To set up a ForeignObjectAssociation, you have to create a ForeignTextField or ForeignPopUp instance in Interface Builder. You then form the association from the controller to the ForeignControl subclass. In the connection inspector select a relationship.
When the association is notified of a change by its controller, it gets the value (an other enterprise object) of the selected enterprise object and sets it as objectValue in the ForeignControl subclass. If the controller has an empty or a multiple selection, the ForeignObjectAssociation instance set objectValue to nil.
When the user changes something in the ForeignControl subclass, the association simply sends an associationDidEdit: message to its controller.
ATTENTION: In this case the controller only sets the relationship to the new object. Before saving the enterprise object back to the database you have to set the join attribute which was used to build up the relationship to the actual value. An ForeignObjectAssociation value is an enterprise object of the entity the relationship points to.
*/
//-------------------------------------------------------------------------------------------

#import "ForeignControlsGlobal.h"
#import "ForeignControlDelegate.h"
#import <eointerface/eointerface.h>

@interface ForeignObjectAssociation : EOAssociation <ForeignControlDelegate>
{
}

//===========================================================================================
//==== ForeignControlDelegate protocol
//===========================================================================================

- (BOOL)controlWillChange:sender;
	// This delegate method is invoked from the foreign control if the control will change its object value

- (void)controlDidChange:sender;
	// This delegate method is invoked from the foreign control if the control did change its object value

//===========================================================================================
//==== Initializing instances
//===========================================================================================

- initWithController:controller key:key destination:destination;
	// Initializes the association, sets itself as the delegate of the ForeignControl subclass to make sure that the association notifies the changes.
	//
	// See also:  - controlWillChange:, - controlDidChange:

//===========================================================================================
//==== Methods invoked by the EOController
//===========================================================================================

- (void)contentsDidChange;
	// Is invoked by the controller and invokes _update.
	
- (void)selectionDidChange;
	// Is invoked by the controller and invokes _update.
	
- value;
	// Returns the objectValue of the ForeignControl subclass. If the objectValue is nil then an EONull instance is returned.

//===========================================================================================
//==== Internal methods
//===========================================================================================

- (void)_update;
	// Gets the value (an enterprise object) of the selected enterprise object and sets it as objectValue in the ForeignControl subclass. If the controller has an empty or a multiple selection the ForeignObjectAssociation instance set objectValue to nil.

- (BOOL)startEditing;
	// Not yet documented.
		
@end
