//-------------------------------------------------------------------------------------------
// Project            ForeignControls.palette
// Class              ForeignPopUp
// Creator            Christian Rapp
// Maintainer         Christian Rapp
// Creation Date      Fri Oct 20 15:39:49 MET 1995
//
// Copyright (C) Uptime Object Factory Inc - All rights reserved.
//
// $Id: ForeignPopUp.h,v 1.1.1.1 1996/03/18 12:53:56 cr Exp $
//-------------------------------------------------------------------------------------------
/* Description:
ForeignPopUp is a subclass of the abstract class ForeignControl. ForeignPopUp contains a popup, and a text field which is optionally used as code field.
ForeignPopUp is especially suitable for small amounts of datasets (<20) using EOF. The ForeignPopUp list is filled up with a database column. That means, that the popup list can dynamically be changed. An other feature of ForeignPopUp is its optional code field. In this code field the user can enter a short cut of one of the values displayed in the popup list and the popup changes to this position (e.g language code). If the code field is enabled then the ForeignPopUp offers all usual features of a TextField like textDelegate, nextText and previousText. (Tab and Shift-Tab is supported as usual) Consequently a ForeignPopUp can be used without using the mouse. 
*/
//-------------------------------------------------------------------------------------------

#import "ForeignControlsGlobal.h"
#import <appkit/appkit.h>
#import <eointerface/eointerface.h>
#import "ForeignControl.h"

#define GET_POPUP_RECT(a)		(NXSetRect((a), 0, 0, 120, 20))
#define GET_CODE_RECT(a)		(NXSetRect((a), 0, 0, 40, 21))

@interface ForeignPopUp : ForeignControl
{
	PopUpList		*popUpList;		// the PopUpList we're associated 
							// with
	NSMutableDictionary	*objectRepository;
	NSComparisonResult	sortOrder;
	NXEvent			endEditingEvent;
}

//===========================================================================================
//==== Creating and freeing instances
//===========================================================================================

- initFrame:(const NXRect*)aRect;
	// Returns self. (Is the designated initializer)
	
- free;
	// Frees the instance

//===========================================================================================
//==== Overridden superclass methods
//===========================================================================================

- updateFromDatabase:sender;
	// Forces a recache of the popup list contents

- setEnabled:(BOOL)flag;
	// Enables or disables the ForeignTextField's ability to receive mouse and keyboard events, according to the value of the flag. Returns self.

//===========================================================================================
//==== Accessor methods
//===========================================================================================

- (void)setPopUpSortOrder:(EOKeySortOrdering *)anOrdering;
	// Sets the sort order of the entries in the popup list. anOrdering is of type EOKeySortOrdering.
	//
	// e.g. : [EOKeySortOrdering keyOrderingWithKey:nil ordering:NSOrderedAscending];
	//
	//NSOrderedAscending (lowest-valued attribute first), NSOrderedDescending (highest-valued attribute first), or NSOrderedSame (no ordering)
	//
	// See also:  - popUpSortOrder

- (EOKeySortOrdering *)popUpSortOrder;
	// Returns a EOKeySortOrdering instance with the current ordering.
	//
	// See also:  - setPopUpSortOrder:
	
//===========================================================================================
//==== Delegate methods
//===========================================================================================

- (BOOL)textWillEnd:textObject;
	// You never invoke this method directly. It is only for internal use.
	
- textDidEnd:sender endChar:(unsigned short)whyEnd;
	// You never invoke this method directly. It is only for internal use.
	
//===========================================================================================
//==== Internal methods
//===========================================================================================

- _delayedEndEditing:sender;
	// You never invoke this method directly. It's only for internal use.

- _popUpChanged:sender;
	// Is invoked from the popup if the user has changed the selection and updates the ForeignPopUp instance. Returns self. You never invoke this method directly. It is only for internal use.
	
- _triggerButtonPressed:sender;
	// Is invoked when the user clicks on the popup button. The very first time when this happens the ForeignPopUp fetches its popup entries from the database. Returns self. You never invoke this method directly. It is only for internal use.
	
- _selectControls:sender;
	// Selects the code field. Returns self. You never invoke this method directly. It is only for internal use.
	
- _updateControls:sender;
	// Updates the code field and the popup after a change. Returns self. You never invoke this method directly. It is only for internal use.

//===========================================================================================
//==== Archiving
//===========================================================================================

- read:(NXTypedStream *)stream;
	// Reads an archived object from a typed stream. Returns self.
	//
	// See also:  - write:
	
- write:(NXTypedStream *)stream;
	// Writes the object to a typed stream. Returns self.
	//
	// See also:  - read:
	

//===========================================================================================
//==== Sorting
//===========================================================================================

int nsStringSort(id eo1, id eo2, void *FCpopUp);
	// Sorting function for popup list sorting. Returns a NSComparisonResult.
	

@end