/*
        Tester.m
	TestSession coordinator.
	(C) Copyright 1993 Zippytech.
	All Rights Reserved.
*/

#import "Tester.h"
#import "Coordinator.h"		/* delegate's class */
#import "CommandScroll.h"

#import <appkit/Matrix.h>
#import <appkit/ButtonCell.h>
#import <appkit/Form.h>
#import <stdio.h>
#import <assert.h>

#import <StreamSession.h>
#import <DatagramSession.h>
#import <DatagramConnSession.h>

void network_error(const char *msg)
{
    NXRunAlertPanel(0, msg, 0, 0, 0);
}

@implementation Tester

-init
{
    [super init];

    session = nil;

    [window makeKeyAndOrderFront:self];
    return self;
}

-displayEndpoints:netsess remote:(BOOL)doremote;
{
    char *lhost, *rhost;
    int lport, rport;
    int sndbuf, rcvbuf;

    lhost = [netsess localAddress];
    lport = [netsess localPort];

    if (doremote) {
	rhost = [netsess peerAddress];
	rport = [netsess peerPort];
    }
    else {
	rhost = malloc(1);
	rport = 0;
    }

    sndbuf = [netsess getSendBufferSize];
    rcvbuf = [netsess getRecvBufferSize];

    [localSelector setStringValue:lhost
		   at:[localSelector findIndexWithTag:0]];
    [localSelector setIntValue:lport
		   at:[localSelector findIndexWithTag:1]];

    [remoteSelector setStringValue:rhost
                    at:[remoteSelector findIndexWithTag:0]];
    [remoteSelector setIntValue:rport
     		    at:[remoteSelector findIndexWithTag:1]];

    [bufSelector setIntValue:sndbuf
     		  at:[bufSelector findIndexWithTag:0]];
    [bufSelector setIntValue:rcvbuf
     		  at:[bufSelector findIndexWithTag:1]];

    return self;
}

/*
 * This method is called when we want to set up a new
 * TCP StreamSession.  It creates a new StreamSession
 * object and updates the displays in the Tester window.
 *
 * Users that are new to the InetObjects classes should
 * look at the calls to SESSION to see how to create and
 * initialize a StreamSession object.
 */
-TCPStreamSession
{
    const char *host, *rhost;
    int port, rport, sndbuf, rcvbuf;

/*
 * Read any values in the displays that the user might
 * have supplied.  We use these to initiale the session
 * object.  After the session object is initialized, we
 * update the displays with the real values.
 */

    host = [localSelector stringValueAt:[localSelector findIndexWithTag:0]];
    port = [localSelector intValueAt:[localSelector findIndexWithTag:1]];
    
    rhost = [remoteSelector stringValueAt:[remoteSelector findIndexWithTag:0]];
    rport = [remoteSelector intValueAt:[remoteSelector findIndexWithTag:1]];

    sndbuf = [bufSelector intValueAt:[bufSelector findIndexWithTag:0]];
    rcvbuf = [bufSelector intValueAt:[bufSelector findIndexWithTag:1]];

#if DEBUG
    fprintf(stderr, "Creating TCP StreamSession\n");
    fprintf(stderr, "\tlocal host=\"%s\" local port=%d\n", host, port);
    fprintf(stderr, "\tremote host=\"%s\" remote port=%d\n", rhost, rport);
#endif


/*
 * Now create and initialize a new StreamSession object.
 */

    session2 = nil;
    session = [[[StreamSession alloc] init] setDelegate:self];
    if (session == nil) {
	network_error("Error creating StreamSession");
	return self;
    }

    /*
     * Set local and remote addresses.
     * If any calls fail, throw away the session.
     * User can restart the test.
     */

    if ([session setLocalAddress:host port:port] == nil) {
	network_error("Error setting StreamSession local address");
	return self;
    }

    if (sndbuf)
	[session setSendBufferSize:sndbuf];
    if (rcvbuf)
	[session setRecvBufferSize:rcvbuf];

    /*
     * This next bit of code is tricky because because it
     * handles both the active ("client") and passive ("server")
     * cases.  If the user doesn't supply a remote host, then
     * the program assumes we are a server and opens the 
     * connection passively.
     *
     * The text thingies in the tester window don't mention
     * that this is possible.  Too hard to explain in such a
     * small amount of space.
     */
    if (rhost[0] != '\0') {
	/* active open -- we are a client */
	if ([session setPeerAddress:rhost port:rport] == nil) {
	    network_error("Error connecting to remote peer");
	    [session free];
	    session = nil;
	}
	session2 = session;	/* open for business */
	[commandScroll appendString:"<connection opened>\n"];
	[self displayEndpoints:session2 remote:YES];
    }
    else {
	/* passive open -- we are a server */
	if ([session passiveOpen] == nil) {
	    network_error("Error in passiveOpen");
	    [session free];
	    session = nil;
	}
	[commandScroll appendString:"<connection passively opened>\n"];
	[self displayEndpoints:session remote:NO];
    }

    return self;
}
    

/*
 * This is the target method of the "GO!" button.
 */
-startTest:sender
{
    id classSelector;
    int tag;

    /*
     * Ignore if a session is in progress.
     */
    if (session != nil)
	return self;

    /*
     * Always open a TCP StreamSession
     */
    [self TCPStreamSession];

#if 0
    /*
     * This is when the tester nib is loaded in response to some
     * higher level selector.  
     */

    /*
     * Figure out what kind of session to create.
     */
    classSelector = [delegate getClassSelector:self];
    tag = [[classSelector selectedCell] tag];

    switch (tag) {
    case 0:
	[self TCPStreamSession];
	break;

    case 1:
	network_error("Sorry, UDP DatagramSession is not supported.\n");
	break;
	
    case 2:
	network_error("Sorry, UDP DatagramConnSession is not supported.\n");
	break;
	
    default:
	network_error("Unknown class tag will be ignored.");
    }
#endif

    return self;
}

/*
 * These methods get called when the user edits the values in
 * the bufsize fields.
 */

- setSendBufSize:sender
{
    int newsize;

    newsize = [bufSelector intValueAt:[bufSelector findIndexWithTag:0]];

    if (session2 != nil) {

	printf("changing send buffer size of %d\n", newsize);
	
	[session2 setSendBufferSize:newsize];
	/*
	 * Refresh display.
	 */
	[self displayEndpoints:session2 remote:YES];
    }
    return self;
}

- setRecvBufSize:sender
{
    int newsize;

    newsize = [bufSelector intValueAt:[bufSelector findIndexWithTag:1]];

    if (session2 != nil) {

	printf("changing recv buffer size of %d\n", newsize);
	
	[session2 setRecvBufferSize:newsize];
	/*
	 * Refresh display.
	 */
	[self displayEndpoints:session2 remote:YES];
    }
    return self;
}


/*
 * Command Scroll delegate methods.
 */
- userEntered:(char *)buffer
{
    int buflen;
    id res;

    if (session2 == nil)	/* nowhere to send it */
	return self;

    buflen = strlen(buffer);

    res = [session2 send:buffer size:&buflen];
    if (res == nil) {
	network_error("Error sending data");
	[session2 free];
	session2 = nil;
    }

    return self;
}
	    


/*
 * Session Delegate methods.
 */

/*
 * Called by StreamSession when data is available for reading.
 */
- readNotify:sender
{
    char buf[512];
    int len, nread, cc;;

#if DEBUG
    fprintf(stderr, "StreamSession 0x%x: reading data\n", session);
#endif

    assert(sender == session2);

    cc = 0;
    while ((nread = [session2 readAvailable]) > 0) {
	len = sizeof(buf);
	if ([sender recv:buf size:&len] == nil) {
	    network_error("Error receiving data");
	    [session2 free];
	    session2 = nil;
	    return self;
	}
	cc += len;
	buf[len] = '\0';

	[commandScroll appendString:buf];
    }	

    /*
     * If there was nothing to read, connection is probably closed.
     */
    if (cc == 0 && nread == 0) {
	[commandScroll appendString:"<connection closed>\n"];
	[session2 free];
	session2 = nil;
    }

    return self;
}

/*
 * Called by StreamSession right before it is freed.
 */
- freeNotify:sender
{
    return self;
}

/*
 * Called by a passively opened StreamSession when a
 * new client has made a connection.  We need to call
 * acceptConnection to create a new StreamSession object
 * corresponding to the new client.
 */
- connReadyNotify:sender
{
    fprintf(stderr, "StreamSession 0x%x: accepting connection\n", session);

    if (session2 != nil) {
	id new_sess;

	/* Accept the connection and then shut it down.
	   If this was a real program, we would probably
	   want to do something more intelligent with it.
	   */
	new_sess = [session acceptConnection];
	if (new_sess != nil)
	    [new_sess free];
	return self;
    }

    session2 = [session acceptConnection];
    if (session2 == nil) {
	network_error("Error accepting connection");
	return self;
    }
    
    [session2 setDelegate:self];

    return self;
}

/*
 * This is called by the StreamSession object when
 * urgent data arrives on the connection.  Urgent
 * data is really only used by the telnet protocol
 * so we basically ignore it here.
 */
- urgentNotify:sender
{
    return self;
}

/*
 * This is called by the StreamSession object when
 * we try to send data on a disconnected connection.
 * Usually if you get this message, it is a sign that
 * something is broken in your program.
 */
- peerDisconnectNotify:sender
{
    return self;
}

/*
 * This is called by the StreamSession object when the
 * session is finished.  We have to clean up our state.
 */
- connFinishedNotify:sender
{
    [commandScroll appendString:"<connection closed>\n"];

    /*
     * Clear local selector.
     */
    [localSelector setStringValue:""
		   at:[localSelector findIndexWithTag:0]];
    [localSelector setStringValue:""
		   at:[localSelector findIndexWithTag:1]];

    if (session == session2) {
	/* Active Open
	   We set session to nil so the user can start another test
	   by pressing the "GO!" button.
	   */	   
	[session free];
	session = session2 = nil;
    }
    else {
	/* passive open */
	[session2 free];
	session2 = nil;

	/* wait for new connection */
    }

    return self;
}

@end
