// TermInterface formal protocol
// Methods to request actions or information
// from the emulator must conform to this protocol.
#import "GlobalDefs.h"

@protocol TermInterface

// terminal emulation methods
// BOOL, returns YES on success, NO on failure.

- (int)cols;  // returns COLS as found in preferences.
- (int)rows;  // returns ROWS as found in preferences.

- (zionPoint)cursorPosition;  // get cursor position..cursor.x == cols, cursor.y == rows
- (BOOL)setCursorPosition:(zionPoint)position;  // set cursor position (0..(ROWS -1)),(0..(COLS -1))

- (BOOL)scrollWindowUp:(zionPoint)scrollRegion;	// scrollRegion.x == top margin, scrollRegion.y == bottom margin
- (BOOL)scrollWindowDown:(zionPoint)scrollRegion;

- (BOOL)readChar:(unsigned char*)aChar attrib:(unsigned char*)aAttrib;  // read character and attrib at cursorPos
- (BOOL)writeChar:(unsigned char)aChar attrib:(unsigned char)aAttrib;  // write character and attrib at cursorPos

- sendText:(unsigned char *)buff;		// forward to dataDelegate which must implement it.
- (BOOL)scrollToBottom;					// scroll on input?
- (BOOL)findText:(const char *)string ignoreCase:(BOOL)ignoreCaseflag backwards:(BOOL)backwardsflag wrap:(BOOL)wrapflag;	// find panel support.
- jumpToSelection:sender;		// scroll buffer to selection.

- termEmulator;	// returns terminal emulator object.
- theFont;		// returns font in use by ScreenView.

// aAttrib is defined as follows:
// 8 bits for attribute (unsigned char)
// 00000000, where highest 000 are foreground RGB
// and next 000 are background RGB,
// and lowest order bits 00 are
// 		00 - normal
// 		01 - intense
// 		10 - underscore
// 		11 - reverse

@end
