// TermInterface formal protocol
// Methods to request actions or information from ScreenView must conform to
// this protocol.
// see TermInterfaceProtocol.rtf for Protocol documentation.
#import "GlobalDefs.h"

@protocol TermInterface

// terminal emulation methods
// BOOL, returns YES on success, NO on failure.

- (int)cols;
- (int)rows;

- (BOOL)monochrome;
- (NXColor)cursorNXColor;
- (NXColor)textNXColor;
- (NXColor)bgNXColor;

- setMonochrome:(BOOL)mono;
- setCursorColor:(NXColor)cursorClr;
- setTextColor:(NXColor)fgClr;
- setBgColor:(NXColor)bgClr;

- (BOOL)scrollBackEnabled;
- (int)scrollBackLength;
- (BOOL)scrollOnInput;
- (BOOL)duplex;
- (zionPoint)screenSize;

- setScrollBackEnabled:(BOOL)status;
- setScrollBackLength:(int)len;
- setScrollOnInput:(BOOL)scrollState;
- setHalfDuplex:(BOOL)half;
- setScreenSize:(zionPoint)size;

// x == cols (0..(COLS -1)), y == rows (0..(ROWS -1))
- (zionPoint)cursorPosition;
- (BOOL)setCursorPosition:(zionPoint)position;

// scrollRegion.x == top margin, scrollRegion.y == bottom margin
- (BOOL)scrollWindowUp:(zionPoint)scrollRegion;
- (BOOL)scrollWindowDown:(zionPoint)scrollRegion;

// character and attrib at cursorPos
- (BOOL)readChar:(unsigned char*)aChar attrib:(unsigned char*)aAttrib;
- (BOOL)writeChar:(unsigned char)aChar attrib:(unsigned char)aAttrib;

// forward to dataDelegate which must implement it.
- sendText:(unsigned char *)buff;

// scroll on input?
- (BOOL)scrollToBottom;

// find panel support
- (BOOL)findText:(const char *)string
		ignoreCase:(BOOL)ignoreCaseflag
		backwards:(BOOL)backwardsflag
		wrap:(BOOL)wrapflag;

// scroll buffer to selection.
- jumpToSelection:sender;

// terminal emulator object.
- termEmulator;
- setTerminal:term;

// returns font in use by ScreenView.
- theFont;			

// aAttrib is defined as follows:
// 8 bits for attribute (unsigned char)
// 00000000, where highest 000 are foreground RGB
// and next 000 are background RGB,
// and lowest order bits 00 are
// 		00 - normal
// 		01 - intense
// 		10 - underscore
// 		11 - reverse

@end
